@@ -4,7 +4,6 @@
 //! - Phase 1: select rollouts, extract stage-1 raw memories, persist stage-1 outputs, and enqueue consolidation.
 //! - Phase 2: claim a global consolidation lock, materialize consolidation inputs, and dispatch one consolidation agent.
 
-mod dispatch;
 mod phase1;
 mod phase2;
 pub(crate) mod prompts;
@@ -26,10 +25,10 @@ mod artifacts {
 
 /// Phase 1 (startup extraction).
 mod phase_one {
+    /// Default model used for phase 1.
+    pub(super) const MODEL: &str = "gpt-5.3-codex-spark";
     /// Prompt used for phase 1.
     pub(super) const PROMPT: &str = include_str!("../../templates/memories/stage_one_system.md");
-    /// Maximum number of rollout candidates processed per startup pass.
-    pub(super) const MAX_ROLLOUTS_PER_STARTUP: usize = 8;
     /// Concurrency cap for startup memory extraction and consolidation scheduling.
     pub(super) const CONCURRENCY_LIMIT: usize = 8;
     /// Fallback stage-1 rollout truncation limit (tokens) when model metadata
@@ -44,10 +43,6 @@ mod phase_one {
     /// Keeping this below 100% leaves room for system instructions, prompt
     /// framing, and model output.
     pub(super) const CONTEXT_WINDOW_PERCENT: i64 = 70;
-    /// Maximum rollout age considered for phase-1 extraction.
-    pub(super) const MAX_ROLLOUT_AGE_DAYS: i64 = 30;
-    /// Minimum rollout idle time required before phase-1 extraction.
-    pub(super) const MIN_ROLLOUT_IDLE_HOURS: i64 = 12;
     /// Lease duration (seconds) for phase-1 job ownership.
     pub(super) const JOB_LEASE_SECONDS: i64 = 3_600;
     /// Backoff delay (seconds) before retrying a failed stage-1 extraction job.
@@ -58,10 +53,8 @@ mod phase_one {
 
 /// Phase 2 (aka `Consolidation`).
 mod phase_two {
-    /// Subagent source label used to identify consolidation tasks.
-    pub(super) const MEMORY_CONSOLIDATION_SUBAGENT_LABEL: &str = "memory_consolidation";
-    /// Maximum number of recent raw memories retained for global consolidation.
-    pub(super) const MAX_RAW_MEMORIES_FOR_GLOBAL: usize = 1_024;
+    /// Default model used for phase 2.
+    pub(super) const MODEL: &str = "gpt-5.3-codex";
     /// Lease duration (seconds) for phase-2 consolidation job ownership.
     pub(super) const JOB_LEASE_SECONDS: i64 = 3_600;
     /// Backoff delay (seconds) before retrying a failed phase-2 consolidation
@@ -76,6 +69,8 @@ mod metrics {
     pub(super) const MEMORY_PHASE_ONE_JOBS: &str = "codex.memory.phase1";
     /// Number of raw memories produced by phase-1 startup extraction.
     pub(super) const MEMORY_PHASE_ONE_OUTPUT: &str = "codex.memory.phase1.output";
+    /// Histogram for aggregate token usage across one phase-1 startup run.
+    pub(super) const MEMORY_PHASE_ONE_TOKEN_USAGE: &str = "codex.memory.phase1.token_usage";
     /// Number of phase-2 startup jobs grouped by status.
     pub(super) const MEMORY_PHASE_TWO_JOBS: &str = "codex.memory.phase2";
     /// Number of stage-1 memories included in each phase-2 consolidation step.
