@@ -22,8 +22,9 @@ fn resume_history(
     let turn_ctx = TurnContextItem {
         turn_id: None,
         cwd: config.cwd.clone(),
-        approval_policy: config.approval_policy.value(),
-        sandbox_policy: config.sandbox_policy.get().clone(),
+        approval_policy: config.permissions.approval_policy.value(),
+        sandbox_policy: config.permissions.sandbox_policy.get().clone(),
+        network: None,
         model: previous_model.to_string(),
         personality: None,
         collaboration_mode: None,
@@ -68,12 +69,19 @@ async fn emits_warning_when_resumed_model_differs() {
         thread: conversation,
         ..
     } = thread_manager
-        .resume_thread_with_history(config, initial_history, auth_manager)
+        .resume_thread_with_history(config, initial_history, auth_manager, false)
         .await
         .expect("resume conversation");
 
     // Assert: a Warning event is emitted describing the model mismatch.
-    let warning = wait_for_event(&conversation, |ev| matches!(ev, EventMsg::Warning(_))).await;
+    let warning = wait_for_event(&conversation, |ev| {
+        matches!(
+            ev,
+            EventMsg::Warning(WarningEvent { message })
+                if message.contains("previous-model") && message.contains("current-model")
+        )
+    })
+    .await;
     let EventMsg::Warning(WarningEvent { message }) = warning else {
         panic!("expected warning event");
     };
