@@ -3,6 +3,7 @@
 mod admin;
 mod config;
 mod http_proxy;
+mod metadata;
 mod network_policy;
 mod policy;
 mod proxy;
@@ -17,7 +18,9 @@ pub use config::NetworkMode;
 pub use config::NetworkProxyConfig;
 pub use config::host_and_port_from_network_addr;
 pub use network_policy::NetworkDecision;
+pub use network_policy::NetworkDecisionSource;
 pub use network_policy::NetworkPolicyDecider;
+pub use network_policy::NetworkPolicyDecision;
 pub use network_policy::NetworkPolicyRequest;
 pub use network_policy::NetworkPolicyRequestArgs;
 pub use network_policy::NetworkProtocol;
@@ -32,6 +35,9 @@ pub use proxy::NetworkProxyHandle;
 pub use proxy::PROXY_URL_ENV_KEYS;
 pub use proxy::has_proxy_url_env_vars;
 pub use proxy::proxy_url_env_value;
+pub use runtime::BlockedRequest;
+pub use runtime::BlockedRequestArgs;
+pub use runtime::BlockedRequestObserver;
 pub use runtime::ConfigReloader;
 pub use runtime::ConfigState;
 pub use runtime::NetworkProxyState;
