@@ -316,7 +316,7 @@ impl RequestUserInputOverlay {
                 // Ensure the selected option is visible in the scroll window.
                 options_state
                     .ensure_visible(option_rows.len(), sections.options_area.height as usize);
-                render_rows(
+                render_rows_bottom_aligned(
                     sections.options_area,
                     buf,
                     &option_rows,
@@ -431,6 +431,48 @@ fn line_width(line: &Line<'_>) -> usize {
         .sum()
 }
 
+/// Render rows into `area`, bottom-aligning the visible rows when fewer than
+/// `area.height` lines are produced.
+///
+/// This keeps footer spacing stable by anchoring the options block to the
+/// bottom of its allocated region.
+fn render_rows_bottom_aligned(
+    area: Rect,
+    buf: &mut Buffer,
+    rows: &[crate::bottom_pane::selection_popup_common::GenericDisplayRow],
+    state: &ScrollState,
+    max_results: usize,
+    empty_message: &str,
+) {
+    if area.width == 0 || area.height == 0 {
+        return;
+    }
+
+    let scratch_area = Rect::new(0, 0, area.width, area.height);
+    let mut scratch = Buffer::empty(scratch_area);
+    for y in 0..area.height {
+        for x in 0..area.width {
+            scratch[(x, y)] = buf[(area.x + x, area.y + y)].clone();
+        }
+    }
+    let rendered_height = render_rows(
+        scratch_area,
+        &mut scratch,
+        rows,
+        state,
+        max_results,
+        empty_message,
+    );
+
+    let visible_height = rendered_height.min(area.height);
+    let y_offset = area.height.saturating_sub(visible_height);
+    for y in 0..visible_height {
+        for x in 0..area.width {
+            buf[(area.x + x, area.y + y_offset + y)] = scratch[(x, y)].clone();
+        }
+    }
+}
+
 /// Truncate a styled line to `max_width`, preferring a word boundary, and append an ellipsis.
 ///
 /// This walks spans character-by-character, tracking the last width-safe position and the last
